package cadprod;

/**
 * Classe que descreve o produto de nossa loja
 * @author prof@caetano.eng.br
 */
public class Produto {
    
    private static int contador = 0;    // Conta objetos criados
    
    private int id;                     // Identificador do Produto
    private int tipo;                   // Tipo de produto (Livro/CD/DVD/BD)
    private String nome;                // Nome do produto
    private double preco;               // Preço do produto em reais
    private String descricaoLonga;      // Descrição completa do produto
    
    private int estoque;                // Quantidade disponível no estoque

    private double desconto;            // Desconto concedido ao produto

    /**
     * Construtor de objetos vazios
     */
    public Produto() {
        this(0,0,"",0.0,"");
    }
    
    /**
     * Construtor-cópia ou Construtor Replicador
     * @param o Objeto a ser copiado
     */
    public Produto( Produto o ) {
        this(o.id, o.tipo, o.nome, o.preco, o.descricaoLonga);
        estoque = o.estoque;
        desconto = o.desconto;
        
    }
    
    /**
     * Construtor do Objeto
     * @param id identificador
     * @param tipo tipo de produto (0-Livro, 1-CD, 2-DVD e 3-BD)
     * @param nome Nome do produto
     * @param preco valor do produto, em reais
     * @param descricaoLonga descrição completa do produto
     */
    public Produto(int id, int tipo, String nome, double preco, String descricaoLonga) {
        this.tipo = tipo;
        this.nome = nome;
        this.preco = preco;
        this.descricaoLonga = descricaoLonga;
        estoque = 0;
        desconto = 0.0;
        contador = contador + 1;
        if (id == 0) this.id = contador;
        else this.id = id;
    }

    /**
     * Descreve o objeto
     * @return Texto contendo a descrição do estado atual do objeto
     */
    @Override
    public String toString() {
        return "*** ID: " + id + " - " + getTipoString() + "\n" + 
                "   " + nome + " - " + descricaoLonga + "\n" + 
                "   Preço: " + getPrecoFormatado() + "\n" +
                "   Estoque: " + getEstoqueString();
    }

    /**
     * Retorna o número de produtos já criados
     * @return número de produtos.
     */
    public static int getContador() {
        return contador;
    }
    
    /**
     * Retorna o número identificador do objeto
     * @return Identificador do objeto
     */
    public int getId() {
        return id;
    }

    /**
     * Modifica o identificador do objeto
     * @param id Novo identificador para o objeto
     * @return true se o valor foi alterado com sucesso 
     */
    public boolean setId(int id) {
        if (id > 0) {
            this.id = id;
            return true;
        }
        System.out.println("ERRO: O idenficiador deve ser maior que zero.");
        return false;
    }

    /**
     * Recupera o tipo de produto, na forma numérica
     * @return Número que identifica o tipo de produto
     *         0- Livro; 1- CD; 2- DVD; 3- BD
     */
    public int getTipo() {
        return tipo;
    }

    /**
     * Altera o tipo de produto
     * @param tipo Número identificador do tipo de objeto
     *         0- Livro; 1- CD; 2- DVD; 3- BD
     * @return true se a alteração do tipo ocorreu com sucesso
     */
    public boolean setTipo(int tipo) {
        if (tipo >= 0 && tipo <= 3) {
            this.tipo = tipo;
            return true;
        }
        System.out.println("ERRO: O tipo deve ser de 0 a 3.");
        return false;
    }

    /**
     * Recupera o tipo de produto, na forma texto
     * @return texto que descreve o tipo de produto.
     */    
    public String getTipoString() {
        switch (tipo) {
            case 0:  return "Livro";
            case 1:  return "CD";
            case 2:  return "DVD";
            case 3:  return "BD";
            default: return "Desconhecido";
        }
    }

    /**
     * Recupera o nome do produto
     * @return Texto que representa o nome do produto.
     */
    public String getNome() {
        return nome;
    }

    /**
     * Muda o nome do produto.
     * @param nome texto com o novo nome do produto
     * @return true se a alteração do nome ocorreu com sucesso
     */
    public boolean setNome(String nome) {
        if (nome.length() > 2) {
            this.nome = nome;
            return true;
        }
        System.out.println("ERRO: O nome deve ter pelo menos três letras!");
        return false;
    }

    /**
     * Recupera o preço do produto
     * @return Valor, em reais, do preço do produto. Se houver desconto, 
     *          ele já estará aplicado ao preço.
     */
    public double getPreco() {
        if (desconto > 0.0) return preco * (100.0 - desconto)/100;
        return preco;
    }
    
    /**
     * Recupera o preço formatado com R$ e duas casas decimais
     * @return Texto com o valor do preço formatado
     */
    public String getPrecoFormatado() {
        return String.format("R$ %.02f",getPreco());
    }

    /**
     * Altera o preço do produto
     * @param preco Valor numérico do novo preço do produto.
     * @return true se a alteração do preço ocorreu com sucesso
     */
    public boolean setPreco(double preco) {
        if (preco >= 0.0) {
            this.preco = preco;
            return true;
        }
        System.out.println("ERRO: O preço não pode ser negativo!");
        return false;
    }

    /**
     * Recupera a descrição longa do produto.
     * @return Texto com a descrição longa do produto.
     */
    public String getDescricaoLonga() {
        return descricaoLonga;
    }

    /**
     * Altera a descrição longa do produto
     * @param descricaoLonga texto com a nova descrição longa do produto
     * @return true se a alteração da descrição longa ocorreu com sucesso
     */
    public boolean setDescricaoLonga(String descricaoLonga) {
        this.descricaoLonga = descricaoLonga;
        return true;
    }

    /** 
     * Recupera valor numérico do estoque
     * @return Quantidade de produto em estoque
     */
    public int getEstoque() {
        return estoque;
    }

    /** 
     * Recupera quantidade em estoque na forma textual
     * @return Texto contento a quantidade em estoque
     */
    public String getEstoqueString() {
        if (estoque > 0) return Integer.toString(estoque);
        return "Indisponível";
    }

    /** 
     * Altera o valor do estoque
     * @param estoque nova quantidade de estoque
     * @return true se a alteração do estoque ocorreu com sucesso
     */
    public boolean setEstoque(int estoque) {
        if (estoque >= 0) {
            this.estoque = estoque;
            return true;
        }
        System.out.println("ERRO: O estoque não pode ser negativo!");
        return false;
    }
    
    
    /**
     * Recupera o valor do desconto do produto
     * @return Valor numérico, de 0 a 100%, do desconto de preço.
     */
    public double getDesconto() {
        return desconto;
    }

    /**
     * Altera o valor do desconto do produto
     * @param desconto novo valor de desconto do produto (0 a 100)
     * @return true se a alteração do valor de desconto ocorreu com sucesso
     */
    public boolean setDesconto(double desconto) {
        if (desconto >= 0.0 && desconto <=100.0) {
            this.desconto = desconto;
            return true;
        }
        System.out.println("ERRO: O desconto deve estar entre 0 e 100!");
        return false;
    }
    
}
